/*
 * Efficiently reads the duration of the high voltage in a pulse train.
 * Designed to simplify the use of RC controllers with the Arduino.
 *
 * Depends on the EnableInterrupt library.
 *
 * Author: David Wang, NuVu Studio
 */

#ifndef __RCPULSEIN_H__
#define __RCPULSEIN_H__


#define EI_ARDUINO_INTERRUPTED_PIN
#include <EnableInterrupt.h>

#define PULSESTART(x) pulseStart ##x
#define PULSEDUR(x) pulseDur ##x

#define defineRC(x) \
  volatile unsigned long PULSESTART(x); \
  volatile unsigned long PULSEDUR(x); \
  void interruptFunction ##x () { \
    if ( arduinoPinState ) { \
      PULSESTART(x)=micros(); \
    } else { \
      PULSEDUR(x)=micros()-PULSESTART(x); \
    } \
  }

#define setupRC(x) \
  pinMode( x, INPUT_PULLUP); \
  enableInterrupt( x, interruptFunction##x, CHANGE)
  
#define readRC(x) PULSEDUR(x)

/*
 * Behaves similarly to the built-in Arduino map function, but maps a "deadband" section of the input range to middle value of the output range.
 * 
 * The return value is constrainted to lie within the range out_min -> out_max.
 * The range of x from in_dead_min to in_dead_max is mapped to the output out_mid.
 */
long deadbandMap(long x, long in_min, long in_dead_min, long in_dead_max, long in_max, long out_min, long out_mid, long out_max);

/*
 * Behaves similarly to the built-in Arduino map function, but maps a "deadband" section of the input range to middle value of the output range.
 * 
 * The return value is constrainted to lie within the range out_min -> out_max.
 * The range of x from in_dead_min to in_dead_max is mapped to the output out_mid.
 */
long deadbandMap(long x, long in_min, long in_deadband, long in_max, long out_min, long out_mid, long out_max);

#endif //__RCPULSEIN_H__